﻿using System;
using System.Linq;
using System.Threading.Tasks;
using System.Collections.ObjectModel;
using System.Net;
using System.IO;
using System.Xml;
using System.ServiceModel.Syndication;
using System.Windows.Media.Imaging;
using System.Threading;
using System.Diagnostics;

namespace Async.Workers
{
	class TheCancellationWay : IDoWork
	{
		private static readonly Uri FlickrRss = new Uri("http://api.flickr.com/services/feeds/photos_public.gne");

		private CancellationTokenSource _cancellationTokenSource;

		public async void DoWork(ObservableCollection<FlickrImage> images)
		{
			_cancellationTokenSource = new CancellationTokenSource();

			try
			{
				WebClient wc = new WebClient();
				string str = await wc.DownloadStringTaskAsync(FlickrRss);
				StringReader sr = new StringReader(str);
				XmlReader reader = XmlReader.Create(sr);
				SyndicationFeed feed = SyndicationFeed.Load(reader);

                FlickrImage[] imageArray = await Task.WhenAll(from item in feed.Items select DownloadItemAsync(item, _cancellationTokenSource.Token));
				foreach(FlickrImage fi in imageArray)
					images.Add(fi);
			}
			catch(WebException)
			{
				Debug.WriteLine("Network error");
			}
			catch(OperationCanceledException)
			{
				Debug.WriteLine("Cancelled");
			}
		}

		private async Task<FlickrImage> DownloadItemAsync(SyndicationItem item, CancellationToken cancellationToken)
		{
            Debug.WriteLine("DownloadItemAsync: " + item.Title.Text);

			Uri uri = (from link in item.Links where link.RelationshipType == "enclosure" select link.Uri).FirstOrDefault();

			WebClient wc = new WebClient();
			cancellationToken.Register(() => wc.CancelAsync());
			byte[] data = await wc.DownloadDataTaskAsync(uri);

			if(cancellationToken.IsCancellationRequested)
				return null;

			//cancellationToken.ThrowIfCancellationRequested();

			MemoryStream ms = new MemoryStream(data);
			BitmapImage bi = new BitmapImage();

			bi.BeginInit();
			bi.StreamSource = ms;
			bi.EndInit();

			FlickrImage fi = new FlickrImage();
			fi.Image = bi;
			fi.Title = item.Title.Text;

			return fi;
		}

		public void Cancel()
		{
			_cancellationTokenSource.Cancel();
		}
	}
}
